@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2021  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -28,6 +28,8 @@
 #include <cassert>
 #include <csignal>
 #include <cstddef>
+#include <string>
+#include <vector>
 
 #include <libnormaliz/dynamic_bitset.h>
 
@@ -53,14 +55,9 @@
 
 #include "libnormaliz/my_omp.h"
 
-
-#ifdef _WIN32     // for 32 and 64 bit windows
-#define NMZ_MPIR  // always use MPIR
-#endif
-
-#ifdef NMZ_MPIR  // use MPIR
+#ifdef USE_MPIR
 #include <mpirxx.h>
-#else  // otherwise use GMP 
+#else  // otherwise use GMP
 #include <gmpxx.h>
 #endif
 
@@ -71,9 +68,17 @@
 
 #ifdef ENFNORMALIZ
 #include <e-antic/renfxx.h>
+namespace libnormaliz {
+using eantic::renf_class;
+using eantic::renf_elem_class;
+typedef boost::intrusive_ptr<const renf_class> renf_class_shared;
+}  // namespace libnormaliz
 #else
+namespace libnormaliz {
 typedef long renf_elem_class;
-typedef long renf_class;
+struct renf_class {};
+typedef renf_class* renf_class_shared;
+}  // namespace libnormaliz
 #endif
 
 namespace libnormaliz {
@@ -87,6 +92,8 @@ const nmz_float nmz_epsilon = 1.0e-12;
 typedef unsigned int key_t;
 
 NORMALIZ_DLL_EXPORT extern bool verbose;
+NORMALIZ_DLL_EXPORT extern bool constructor_verbose;
+NORMALIZ_DLL_EXPORT extern bool polynomial_verbose;
 NORMALIZ_DLL_EXPORT extern size_t GMP_mat, GMP_hyp, GMP_scal_prod;
 NORMALIZ_DLL_EXPORT extern size_t TotDet;
 
@@ -94,6 +101,7 @@ NORMALIZ_DLL_EXPORT extern bool int_max_value_dual_long_computed;
 NORMALIZ_DLL_EXPORT extern bool int_max_value_dual_long_long_computed;
 NORMALIZ_DLL_EXPORT extern bool int_max_value_primary_long_computed;
 NORMALIZ_DLL_EXPORT extern bool int_max_value_primary_long_long_computed;
+NORMALIZ_DLL_EXPORT extern bool no_output_on_interrupt;
 
 #ifdef NMZ_EXTENDED_TESTS
 NORMALIZ_DLL_EXPORT extern bool test_arith_overflow_full_cone, test_arith_overflow_dual_mode;
@@ -120,13 +128,19 @@ NORMALIZ_DLL_EXPORT extern volatile sig_atomic_t nmz_interrupted;
 // extern bool test_arithmetic_overflow;
 // extern long overflow_test_modulus;
 
-NORMALIZ_DLL_EXPORT extern long default_thread_limit;
-NORMALIZ_DLL_EXPORT extern long thread_limit;
+NORMALIZ_DLL_EXPORT extern const int default_thread_limit;
+NORMALIZ_DLL_EXPORT extern int thread_limit;
 NORMALIZ_DLL_EXPORT extern bool parallelization_set;
-long set_thread_limit(long t);
+int set_thread_limit(int t);
+
+// debugging helpers
+NORMALIZ_DLL_EXPORT extern long cone_recursion_level;
+NORMALIZ_DLL_EXPORT extern long full_cone_recursion_level;
 
 /* set the verbose default value */
 bool setVerboseDefault(bool v);
+void suppressNextConstructorVerbose();
+bool setPolynomialVerbose(bool onoff);
 /* methods to set and use the output streams */
 void setVerboseOutput(std::ostream&);
 void setErrorOutput(std::ostream&);
@@ -136,6 +150,26 @@ std::ostream& errorOutput();
 
 void interrupt_signal_handler(int signal);
 
+void StartTime(struct timeval& var_TIME_begin);
+void StartGlobalTime();
+void StartTime();
+double MeasureTime(const struct timeval var_TIME_begin);
+void MeasureTime(bool verbose, const std::string& step);
+double TimeSinceStart();
+void MeasureGlobalTime(bool verbose);
+void PrintTime(const struct timeval var_TIME_begin, bool verbose, const std::string& step);
+
+
+NORMALIZ_DLL_EXPORT extern double GlobalTimeBound;
+NORMALIZ_DLL_EXPORT extern double GlobalPredictionTimeBound;
+
+NORMALIZ_DLL_EXPORT extern std::vector<long> split_patches;
+NORMALIZ_DLL_EXPORT extern std::vector<long> split_moduli;
+NORMALIZ_DLL_EXPORT extern std::vector<long> split_residues;
+NORMALIZ_DLL_EXPORT extern long split_res;
+
+NORMALIZ_DLL_EXPORT extern std::string global_project;
+
 } /* end namespace libnormaliz */
 
 #include <libnormaliz/normaliz_exception.h>
